@@ -1,5 +1,6 @@
 /* Atomix -- a little puzzle game about atoms and molecules.
  * Copyright (C) 1999-2001 Jens Finke
+ * Copyright (C) 2005 Guilherme de S. Pastore
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,7 +32,7 @@
 #include "goal.h"
 #include "level.h"
 #include "goal-view.h"
-#include "gtk-clock.h"
+#include "clock.h"
 #include "undo.h"
 
 AtomixApp *app;
@@ -48,20 +49,19 @@ typedef enum
     GAME_ACTION_RESTART,
   } GameAction;
 
-static void controller_handle_action (AtomixApp *app, GameAction action);
-static void set_game_not_running_state (AtomixApp *app);
-static gboolean set_next_level (AtomixApp *app);
-static void setup_level (AtomixApp *app);
-static void level_cleanup_view (AtomixApp *app);
-static void atomix_exit (AtomixApp *app);
+static void controller_handle_action (GameAction action);
+static void set_game_not_running_state (void);
+static gboolean set_next_level (void);
+static void setup_level (void);
+static void level_cleanup_view (void);
+static void atomix_exit (void);
 static gboolean on_key_press_event (GObject *widget, GdkEventKey *event,
 				    gpointer user_data);
-static void game_init (AtomixApp *app);
-static void update_statistics (AtomixApp *app);
-void update_menu_item_state (AtomixApp *aapp);
+static void game_init (void);
+static void update_statistics (void);
 static void view_congratulations (void);
-static void calculate_score (AtomixApp *app);
-static void log_score (AtomixApp *app);
+static void calculate_score (void);
+static void log_score (void);
 
 /* ===============================================================
       
@@ -71,43 +71,43 @@ static void log_score (AtomixApp *app);
 static void verb_GameNew_cb (BonoboUIComponent *uic, gpointer user_data,
 			     const char *cname)
 {
-  controller_handle_action ((AtomixApp *) user_data, GAME_ACTION_NEW);
+  controller_handle_action (GAME_ACTION_NEW);
 }
 
 static void verb_GameEnd_cb (BonoboUIComponent * uic, gpointer user_data,
 			     const char *cname)
 {
-  controller_handle_action ((AtomixApp *) user_data, GAME_ACTION_END);
+  controller_handle_action (GAME_ACTION_END);
 }
 
 static void verb_GameSkip_cb (BonoboUIComponent * uic, gpointer user_data,
 			      const char *cname)
 {
-  controller_handle_action ((AtomixApp *) user_data, GAME_ACTION_SKIP);
+  controller_handle_action (GAME_ACTION_SKIP);
 }
 
 static void verb_GameReset_cb (BonoboUIComponent * uic, gpointer user_data,
 			       const char *cname)
 {
-  controller_handle_action ((AtomixApp *) user_data, GAME_ACTION_RESTART);
+  controller_handle_action (GAME_ACTION_RESTART);
 }
 
 static void verb_GamePause_cb (BonoboUIComponent * uic, gpointer user_data,
 			       const char *cname)
 {
-  controller_handle_action ((AtomixApp *) user_data, GAME_ACTION_PAUSE);
+  controller_handle_action (GAME_ACTION_PAUSE);
 }
 
 static void verb_GameContinue_cb (BonoboUIComponent * uic,
 				  gpointer user_data, const char *cname)
 {
-  controller_handle_action ((AtomixApp *) user_data, GAME_ACTION_CONTINUE);
+  controller_handle_action (GAME_ACTION_CONTINUE);
 }
 
 static void verb_GameUndo_cb (BonoboUIComponent * uic, gpointer user_data,
 			      const char *cname)
 {
-  controller_handle_action ((AtomixApp *) user_data, GAME_ACTION_UNDO);
+  controller_handle_action (GAME_ACTION_UNDO);
 }
 
 static void verb_GameScores_cb (BonoboUIComponent * uic, gpointer user_data,
@@ -138,7 +138,7 @@ static void verb_GameScores_cb (BonoboUIComponent * uic, gpointer user_data,
 static void verb_GameExit_cb (BonoboUIComponent * uic, gpointer user_data,
 			      const char *cname)
 {
-  atomix_exit ((AtomixApp *) user_data);
+  atomix_exit ();
 }
 
 static void verb_EditPreferences_cb (BonoboUIComponent * uic,
@@ -156,7 +156,7 @@ static void verb_HelpAbout_cb (BonoboUIComponent *uic, gpointer user_data,
 
   const char *authors[] =
     {
-      "Guilherme de S. Pastore <guilherme.pastore@terra.com.br>",
+      "Guilherme de S. Pastore <gpastore@gnome.org>",
       "Jens Finke <jens@triq.net>",
       NULL
     };
@@ -182,8 +182,7 @@ static void verb_HelpAbout_cb (BonoboUIComponent *uic, gpointer user_data,
 static gboolean on_app_destroy_event (GtkWidget *widget, GdkEvent *event,
 				      gpointer user_data)
 {
-  AtomixApp *app = (AtomixApp *) user_data;
-  atomix_exit (app);
+  atomix_exit ();
 
   return TRUE;
 }
@@ -194,18 +193,18 @@ static gboolean on_app_destroy_event (GtkWidget *widget, GdkEvent *event,
 
 -------------------------------------------------------------- */
 
-static void controller_handle_action (AtomixApp *app, GameAction action)
+static void controller_handle_action (GameAction action)
 {
   switch (app->state)
     {
     case GAME_STATE_NOT_RUNNING:
       if (action == GAME_ACTION_NEW)
 	{
-	  if (set_next_level (app))
+	  if (set_next_level ())
 	    {
 	      app->level_no = 1;
 	      app->score = 0;
-	      setup_level (app);
+	      setup_level ();
 	      app->state = GAME_STATE_RUNNING_UNMOVED;
 	    }
 	}
@@ -216,50 +215,50 @@ static void controller_handle_action (AtomixApp *app, GameAction action)
       switch (action)
 	{
 	case GAME_ACTION_END:
-	  log_score (app);
-	  level_cleanup_view (app);
-	  set_game_not_running_state (app);
+	  log_score ();
+	  level_cleanup_view ();
+	  set_game_not_running_state ();
 	  break;
 
 	case GAME_ACTION_PAUSE:
-	  gtk_clock_stop (GTK_CLOCK (app->clock));
+	  clock_stop (CLOCK (app->clock));
 	  board_hide ();
 	  app->state = GAME_STATE_PAUSED;
 	  break;
 
 	case GAME_ACTION_SKIP:
-	  level_cleanup_view (app);
+	  level_cleanup_view ();
 
-	  if (set_next_level (app))
-	    setup_level (app);
+	  if (set_next_level ())
+	    setup_level ();
 
 	  else
-	    set_game_not_running_state (app);
+	    set_game_not_running_state ();
 
 	  break;
 
 	case GAME_ACTION_FINISHED:
-	  calculate_score (app);
+	  calculate_score ();
 	  if (level_manager_is_last_level (app->lm, app->level))
 	    {
 	      view_congratulations ();
-	      log_score (app);
-	      level_cleanup_view (app);
-	      set_game_not_running_state (app);
+	      log_score ();
+	      level_cleanup_view ();
+	      set_game_not_running_state ();
 	    }
 	  else
 	    {
-	      level_cleanup_view (app);
-	      set_next_level (app);
-	      setup_level (app);
+	      level_cleanup_view ();
+	      set_next_level ();
+	      setup_level ();
 	    }
 	  break;
 
 	case GAME_ACTION_RESTART:
 	  g_assert (app->state != GAME_STATE_RUNNING_UNMOVED);
 
-	  level_cleanup_view (app);
-	  setup_level (app);
+	  level_cleanup_view ();
+	  setup_level ();
 	  break;
 
 	case GAME_ACTION_UNDO:
@@ -276,7 +275,7 @@ static void controller_handle_action (AtomixApp *app, GameAction action)
     case GAME_STATE_PAUSED:
       if (action == GAME_ACTION_CONTINUE)
 	{
-	  gtk_clock_start (GTK_CLOCK (app->clock));
+	  clock_start (CLOCK(app->clock));
 	  board_show ();
 	  app->state = (undo_exists())?GAME_STATE_RUNNING:GAME_STATE_RUNNING_UNMOVED;
 	}
@@ -286,11 +285,11 @@ static void controller_handle_action (AtomixApp *app, GameAction action)
       g_assert_not_reached ();
     }
 
-  update_menu_item_state (app);
-  update_statistics (app);
+  update_menu_item_state ();
+  update_statistics ();
 }
 
-static void set_game_not_running_state (AtomixApp *app)
+static void set_game_not_running_state (void)
 {
   board_show_logo (TRUE);
 
@@ -306,7 +305,7 @@ static void set_game_not_running_state (AtomixApp *app)
   app->state = GAME_STATE_NOT_RUNNING;
 }
 
-static gboolean set_next_level (AtomixApp *app)
+static gboolean set_next_level (void)
 {
   Level *next_level;
   PlayField *goal_pf;
@@ -340,7 +339,7 @@ static gboolean set_next_level (AtomixApp *app)
   return TRUE;
 }
 
-static void setup_level (AtomixApp *app)
+static void setup_level (void)
 {
   PlayField *env_pf;
   PlayField *sce_pf;
@@ -362,22 +361,22 @@ static void setup_level (AtomixApp *app)
   goal_view_render (app->goal);
 
   /* init clock */
-  gtk_clock_set_seconds (GTK_CLOCK (app->clock), 0);
-  gtk_clock_start (GTK_CLOCK (app->clock));
+  clock_set_seconds (CLOCK(app->clock), 0);
+  clock_start (CLOCK(app->clock));
 
   g_object_unref (env_pf);
   g_object_unref (sce_pf);
 }
 
-static void level_cleanup_view (AtomixApp *app)
+static void level_cleanup_view (void)
 {
   board_clear ();
   goal_view_clear ();
 
-  gtk_clock_stop (GTK_CLOCK (app->clock));
+  clock_stop (CLOCK(app->clock));
 }
 
-static void atomix_exit (AtomixApp *app)
+static void atomix_exit (void)
 {
 #ifdef DEBUG
   g_message ("Destroy application");
@@ -387,8 +386,8 @@ static void atomix_exit (AtomixApp *app)
 
   if (app->state != GAME_STATE_NOT_RUNNING)
     {
-      log_score (app);
-      set_game_not_running_state (app);
+      log_score ();
+      set_game_not_running_state ();
     }
 
   board_destroy ();
@@ -415,8 +414,6 @@ static void atomix_exit (AtomixApp *app)
 static gboolean on_key_press_event (GObject *widget, GdkEventKey *event,
 				    gpointer user_data)
 {
-  AtomixApp *app = (AtomixApp *) user_data;
-
   if ((app->state == GAME_STATE_RUNNING) || (app->state == GAME_STATE_RUNNING_UNMOVED))
     {
       board_handle_key_event (NULL, event, NULL);
@@ -425,7 +422,7 @@ static gboolean on_key_press_event (GObject *widget, GdkEventKey *event,
   return TRUE;
 }
 
-static void game_init (AtomixApp *app)
+static void game_init ()
 {
   g_return_if_fail (app != NULL);
 
@@ -443,8 +440,8 @@ static void game_init (AtomixApp *app)
   app->level = NULL;
   app->level_no = 0;
   app->score = 0;
-  gtk_clock_set_format (GTK_CLOCK (app->clock), "%M:%S");
-  gtk_clock_set_seconds (GTK_CLOCK (app->clock), 0);
+  clock_set_format (CLOCK(app->clock), "%M:%S");
+  clock_set_seconds (CLOCK(app->clock), 0);
 
   /* init the board */
   board_init (app->theme, GNOME_CANVAS (app->ca_matrix));
@@ -454,22 +451,22 @@ static void game_init (AtomixApp *app)
 
   /* update user visible information */
   app->state = GAME_STATE_NOT_RUNNING;
-  update_menu_item_state (app);
-  update_statistics (app);
+  update_menu_item_state ();
+  update_statistics ();
 
   gtk_widget_grab_focus (GTK_WIDGET (app->ca_matrix));
 }
 
-void game_level_finished (AtomixApp *tmp_app)
+void game_level_finished (void)
 {
-  controller_handle_action (app, GAME_ACTION_FINISHED);
+  controller_handle_action (GAME_ACTION_FINISHED);
 }
 
-static void calculate_score (AtomixApp *app)
+static void calculate_score (void)
 {
   gint seconds;
 
-  seconds = time (NULL) - GTK_CLOCK (app->clock)->seconds;
+  seconds = time (NULL) - CLOCK(app->clock)->seconds;
 
   if (seconds > 300)
     return;
@@ -480,7 +477,7 @@ static void calculate_score (AtomixApp *app)
     app->score = app->score * (2 - (seconds / 300));
 }
 
-static void log_score (AtomixApp *app)
+static void log_score (void)
 {
   gint position;
 
@@ -522,7 +519,7 @@ _("Do you want to finish the game?"));
              UI update functions
 
 -------------------------------------------------------------- */
-static void update_statistics (AtomixApp *app)
+static void update_statistics (void)
 {
   gchar *str_buffer;
 
@@ -533,6 +530,7 @@ static void update_statistics (AtomixApp *app)
       /* don't show anything */
       gtk_label_set_text (GTK_LABEL (app->lb_level), "");
       gtk_label_set_text (GTK_LABEL (app->lb_name), "");
+      gtk_label_set_text (GTK_LABEL (app->lb_formula), "");
       gtk_label_set_text (GTK_LABEL (app->lb_score), "");
       gtk_widget_hide (GTK_WIDGET (app->clock));
     }
@@ -547,6 +545,10 @@ static void update_statistics (AtomixApp *app)
       gtk_label_set_text (GTK_LABEL (app->lb_name),
 			  _(level_get_name (app->level)));
 
+      /* set the formula of the compound */
+      gtk_label_set_markup (GTK_LABEL (app->lb_formula),
+			    level_get_formula (app->level));
+
       /* set score */
       g_snprintf (str_buffer, 10, "%i", app->score);
       gtk_label_set_text (GTK_LABEL (app->lb_score), str_buffer);
@@ -621,7 +623,7 @@ static const CmdEnable *state_sensitivity[] =
     not_running, running_unmoved, running, paused
   };
 
-void update_menu_item_state (AtomixApp *app)
+void update_menu_item_state (void)
 {
   gchar *path;
   gint i;
@@ -687,7 +689,7 @@ static void add_statistics_table_entry (GtkWidget *table, gint row,
 		    0, 1, row, row + 1, GTK_FILL, GTK_FILL, 0, 0);
 
   if (is_clock)
-    *return_widget = gtk_clock_new (GTK_CLOCK_INCREASING);
+    *return_widget = clock_new ();
   else
     *return_widget = gtk_label_new ("NO CONTENT");
 
@@ -731,8 +733,9 @@ static GtkWidget *create_mainwin_content (AtomixApp *app)
 
   add_statistics_table_entry (table, 0, _("Level:"), FALSE, &app->lb_level);
   add_statistics_table_entry (table, 1, _("Molecule:"), FALSE, &app->lb_name);
-  add_statistics_table_entry (table, 2, _("Score:"), FALSE, &app->lb_score);
-  add_statistics_table_entry (table, 3, _("Time:"), TRUE, &app->clock);
+  add_statistics_table_entry (table, 2, _("Formula:"), FALSE, &app->lb_formula);
+  add_statistics_table_entry (table, 3, _("Score:"), FALSE, &app->lb_score);
+  add_statistics_table_entry (table, 4, _("Time:"), TRUE, &app->clock);
 
   gtk_container_add (GTK_CONTAINER (frame), GTK_WIDGET (table));
 
@@ -814,7 +817,7 @@ int main (int argc, char *argv[])
   /* make a few initalisations here */
   app = create_gui (prog);
 
-  game_init (app);
+  game_init ();
 
   gtk_widget_set_size_request (GTK_WIDGET (app->mainwin), 660, 480);
   gtk_widget_show (app->mainwin);
